import pandas as pd
import re
import copy
from datetime import datetime
from unidiff import PatchSet
import urllib.request

commits_to_check = 'FILTER_SHEETNAME_EXPLANATION_annotate_maxpat_files_commit_links_2023_05_10-02_41_48_AM.csv.csv - 3_sort_by_fixingcommit.csv'

sha_regex = r"\b[0-9a-f]{5,40}\b"

valid_file_extensions = ['maxpat', 'maxhelp'] 

df = pd.read_csv(commits_to_check)

search_time = datetime.now().strftime("%Y_%m_%d-%I_%M_%S_%p")

def check_for_files(row):
    repo_name = '/'.join(row['html_url'].split('/')[-4:-2])

    fn_count = 0
    if isinstance(row['fixing_commit'], str):
        row['fixing_commit'] = row['fixing_commit'].strip()
        fixing_commit = row['fixing_commit']

        if not fixing_commit:
            return row

        diff = urllib.request.urlopen(f"https://github.com/{repo_name}/commit/{fixing_commit}.diff")
        encoding = diff.headers.get_charsets()[0]
        patch = PatchSet(diff, encoding=encoding)

        
        for p in patch.modified_files + patch.added_files:
            filename = p.path
            filename_ext = filename.split('.')[-1]

            if filename_ext in valid_file_extensions:
                    fn_count+=1
    
        row['fixing_commit_files_count'] = len(patch.modified_files + patch.added_files)
        row['fixing_commit_files_valid_count'] = fn_count

    return row

df = df.apply(check_for_files, axis = 1)

# Keep only commits that modify 1 file, and keep fixing commits that appear once (commit could fix multiple issues)
df[df['fixing_commit_files_count'] == 1].drop_duplicates(subset='fixing_commit', keep=False).to_csv(f'fixing_commits_file_count_maxpat_files_commit_links_{search_time}.csv', index=False)