import requests
from bs4 import BeautifulSoup
import pandas as pd
import re
import copy
from datetime import datetime
from unidiff import PatchSet
import urllib.request

commits_to_check = 'commit_links_pd_issues_2023_05_09-09_23_47_PM.csv'

sha_regex = r"\b[0-9a-f]{5,40}\b"

valid_file_extensions = ['pd'] 

df = pd.read_csv(commits_to_check)
df['commit_links'] = df['commit_links'].apply(lambda x: eval(x))

search_time = datetime.now().strftime("%Y_%m_%d-%I_%M_%S_%p")

def check_for_files(row):
    repo_name = '/'.join(row['repository'].split('/')[-2:])

    print(repo_name)

    shas = set()

    for link in row['commit_links']:
        if repo_name in link:
            shas.update(re.findall(sha_regex, link))

    row['shas'] = shas

    shas_containing_file = copy.deepcopy(shas)

    for s in shas:
        try:
            diff = urllib.request.urlopen(f"https://github.com/{repo_name}/commit/{s}.diff")
            encoding = diff.headers.get_charsets()[0]
            patch = PatchSet(diff, encoding=encoding)
        except Exception as e:
            print(e)
            continue
        
        hit = False
        for p in patch.modified_files:
            filename = p.path
            filename_ext = filename.split('.')[-1]

            if filename_ext in valid_file_extensions:
                hit = True
            
        if not hit:
            shas_containing_file.discard(s)

    row['shas_containing_file'] = shas_containing_file

    return row

df = df[df['commit_links'].astype(bool)]

df = df.apply(check_for_files, axis = 1)

df.to_csv(f'pd_files_commit_links_{search_time}.csv', index=False)