from pydriller import ModificationType
from git import GitCommandError
import re
import logging as logger
import pprint

def textual_szz(commit, gr, debug=False):
    # This implementation is based off the SZZ implementation from PyDriller
    # This SZZ is specifically adapted for JSON files

    commit = gr.get_commit(commit)

    commits = {}

    import itertools
    def to_ranges(iterable):
        iterable = sorted(set(iterable))
        for key, group in itertools.groupby(enumerate(iterable),
                                            lambda t: t[1] - t[0]):
            group = list(group)
            yield group[0][1], group[-1][1]

    def useless_line(line: str):
        # this ignores json lines that are brackets
        brackets = ['{', '}', '[', ']']
        comma_with_brackets = [','] + [',' + b for b in brackets] + ['],', '},']

        replaced_line = re.sub(r"\s+", "", line)
        
        return not line or \
            line in brackets or \
            replaced_line in brackets or \
            replaced_line in comma_with_brackets

    for mod in commit.modified_files:
        path = mod.new_path
        if mod.change_type == ModificationType.RENAME or mod.change_type == ModificationType.DELETE:
            path = mod.old_path

        deleted_lines = mod.diff_parsed['deleted']

        assert path is not None, "We could not find the path to the file"
        
        commit_blamed_lines = {}

        try:
            blame = gr._get_blame(commit.hash, path, None)
            for num_line, line in deleted_lines:
                if not useless_line(line.strip()):
                    buggy_commit = blame[num_line - 1].split(' ')[0].replace('^', '')

                    # Skip unblamable lines.
                    if buggy_commit.startswith("*"):
                        continue

                    if mod.change_type == ModificationType.RENAME:
                        path = mod.new_path

                    assert path is not None, "We could not find the path to the file"

                    if buggy_commit not in commit_blamed_lines:
                        commit_blamed_lines[buggy_commit] = [num_line]
                    else:
                        commit_blamed_lines[buggy_commit].append(num_line)

                    commits.setdefault(path, set()).add(gr.get_commit(buggy_commit).hash)
        except GitCommandError:
            logger.debug(f"Could not find file {mod.filename} in commit {commit.hash}. Probably a double rename!")

    if debug:
        commit_blamed_lines.update((k,list(to_ranges(v))) for k,v in commit_blamed_lines.items())
        pprint.pprint(commit_blamed_lines)
        # return commit_blamed_lines
    
    return commits