from git import Repo
import pandas as pd
import os
from pydriller import Git, Repository
import subprocess
import pprint
from statistics import mean

import sys
original_path = sys.path
sys.path.append("..") 
import importlib
textual_szz = importlib.import_module('Visual-Code-SZZ.szz.textual_szz')
textual_szz = textual_szz.textual_szz
szz_vc = importlib.import_module('Visual-Code-SZZ.szz.szz_vc')
szz_vc = szz_vc.szz_vc
sys.path = original_path

max_path = 'data_retrieval/max/SELECTED_fixing_commits_file_count_maxpat_files_commit_links_2023_05_11-06_01_40_AM.csv'

max_ext = ['maxpat', 'maxhelp']

def get_short_sha(long_sha):
    return subprocess.check_output(['git', 'rev-parse', '--short', long_sha]).decode('utf-8').strip()

def avg_modified_count(fixing_sha, gr):
    fix_commit = gr.get_commit(fixing_sha)

    fixing_sha = fix_commit.hash

    filepaths = [f.new_path for f in fix_commit.modified_files if f is not None]

    counts = []

    for f in filepaths:
        commits = [fix_commit]
        for commit in Repository(gr.repo.working_tree_dir, filepath=f).traverse_commits():
            commits.append(commit)

        counts.append(len(commits))
    
    return mean(counts)

df1 = pd.read_csv(max_path)
df1['language'] = 'max'

issues = df1

repos = set()

textual_szz_results = []

szz_vc_depth_2_results = []
szz_vc_max_depth_results = []

for i, row in issues[-1:].iterrows():
    print(row['html_url'])
    repo_name = '/'.join(row['html_url'].split('/')[-4:-2])
    repo_path = f"validation/repos/{repo_name}"

    if not os.path.exists(repo_path):
        print(repo_name)
        Repo.clone_from(f"https://github.com/{repo_name}.git", repo_path)

    gr = Git(repo_path)
    main_branch = gr.repo.branches[0].name
    fixing_sha = row['fixing_commit'].strip()

    # if avg_modified_count(fixing_sha, gr) > 20:
    #     # In the case of a single file changed per commmit, if there are more than 50 commits modifying the file skip it
    #     continue

    buggy_commits_szz = textual_szz(fixing_sha, gr, True)

    for k,v in buggy_commits_szz.items():
        if row['language'] == 'pd' and k.split('.')[-1] in pd_ext:
            for c in v:
                textual_szz_results.append({
                    'language': row['language'],
                    'repo': repo_name,
                    'html_url': row['html_url'],
                    'fixing_commit': fixing_sha,
                    'potential_inducing_commit': get_short_sha(c),
                    'classification': '',
                    'comment': ''
                })

        elif row['language'] == 'max' and k.split('.')[-1] in max_ext:
            for c in v:
                 textual_szz_results.append({
                    'language': row['language'],
                    'repo': repo_name,
                    'html_url': row['html_url'],
                    'fixing_commit': fixing_sha,
                    'potential_inducing_commit': get_short_sha(c),
                    'classification': '',
                    'comment': ''
                })

    # print(repo_name, row['html_url'], fixing_sha)
    buggy_commits_szz_vc = szz_vc(fixing_sha, main_branch, gr, row['language'], True, cache_key=repo_name)

    for k,v in buggy_commits_szz_vc['max'].items():
        if row['language'] == 'pd' and k.split('.')[-1] in pd_ext:
            for c in v:
                szz_vc_max_depth_results.append({
                    'language': row['language'],
                    'repo': repo_name,
                    'html_url': row['html_url'],
                    'fixing_commit': fixing_sha,
                    'potential_inducing_commit': get_short_sha(c),
                    'classification': '',
                    'comment': ''
                })

        elif row['language'] == 'max' and k.split('.')[-1] in max_ext:
            for c in v:
                 szz_vc_max_depth_results.append({
                    'language': row['language'],
                    'repo': repo_name,
                    'html_url': row['html_url'],
                    'fixing_commit': fixing_sha,
                    'potential_inducing_commit': get_short_sha(c),
                    'classification': '',
                    'comment': ''
                })

    for k,v in buggy_commits_szz_vc['one'].items():
        if row['language'] == 'pd' and k.split('.')[-1] in pd_ext:
            for c in v:
                szz_vc_depth_2_results.append({
                    'language': row['language'],
                    'repo': repo_name,
                    'html_url': row['html_url'],
                    'fixing_commit': fixing_sha,
                    'potential_inducing_commit': get_short_sha(c),
                    'classification': '',
                    'comment': ''
                })

        elif row['language'] == 'max' and k.split('.')[-1] in max_ext:
            for c in v:
                 szz_vc_depth_2_results.append({
                    'language': row['language'],
                    'repo': repo_name,
                    'html_url': row['html_url'],
                    'fixing_commit': fixing_sha,
                    'potential_inducing_commit': get_short_sha(c),
                    'classification': '',
                    'comment': ''
                })

    repos.add(repo_name)

pd.DataFrame(textual_szz_results).to_csv('validation/annotate_textual_szz_max.csv', index=False)
pd.DataFrame(szz_vc_depth_2_results).to_csv('validation/annotate_szz_vc_depth_2_max.csv', index=False)
pd.DataFrame(szz_vc_max_depth_results).to_csv('validation/annotate_szz_vc_max_depth_max.csv', index=False)